@@ -136,7 +136,11 @@ class DtaleFlask(Flask):
     def url_for(self, endpoint, *args, **kwargs):
         if self.app_root is not None and endpoint == "static":
             if "filename" in kwargs:
-                return fix_url_path("{}/{}".format(self.app_root, kwargs["filename"]))
+                return fix_url_path(
+                    "{}/{}/{}".format(
+                        self.app_root, self.static_url_path, kwargs["filename"]
+                    )
+                )
             return fix_url_path("{}/{}".format(self.app_root, args[0]))
         return url_for(endpoint, *args, **kwargs)
 
@@ -271,7 +275,7 @@ def build_app(
     app = DtaleFlask(
         "dtale",
         reaper_on=reaper_on,
-        static_url_path="",
+        static_url_path="/dtale/static",
         url=url,
         instance_relative_config=False,
         app_root=app_root,
@@ -674,7 +678,11 @@ def show(data=None, data_loader=None, name=None, context_vars=None, **options):
             allow_cell_edits=final_options["allow_cell_edits"],
             inplace=final_options["inplace"],
             drop_index=final_options["drop_index"],
+            precision=final_options["precision"],
+            show_columns=final_options["show_columns"],
+            hide_columns=final_options["hide_columns"],
         )
+        instance.started_with_open_browser = final_options["open_browser"]
         is_active = not running_with_flask_debug() and is_up(app_url)
         if is_active:
 
