@@ -76,8 +76,8 @@ def valid_chart(chart_type=None, x=None, y=None, z=None, **inputs):
         treemap_props = ["treemap_value", "treemap_label"]
         return all(inputs.get(p) is not None for p in treemap_props)
 
-    if x is None or not len(y or []):
-        return False
+    if not y:
+        return chart_type == "wordcloud" and inputs.get("agg") == "count"
 
     if chart_type in ZAXIS_CHARTS and z is None:
         return False
@@ -89,7 +89,7 @@ def valid_chart(chart_type=None, x=None, y=None, z=None, **inputs):
     return True
 
 
-def build_formatters(df):
+def build_formatters(df, nan_display=None):
     """
     Helper around :meth:`dtale.utils.grid_formatters` that will build a formatter for the data being fed into a chart as
     well as a formatter for the min/max values for each column used in the chart data.
@@ -100,9 +100,9 @@ def build_formatters(df):
     :rtype: (:class:`dtale.utils.JSONFormatter`, :class:`dtale.utils.JSONFormatter`)
     """
     cols = grid_columns(df)
-    data_f = grid_formatter(cols, nan_display=None)
+    data_f = grid_formatter(cols, nan_display=nan_display)
     overrides = {"F": lambda f, i, c: f.add_float(i, c, precision=2)}
-    range_f = grid_formatter(cols, overrides=overrides, nan_display=None)
+    range_f = grid_formatter(cols, overrides=overrides, nan_display=nan_display)
     return data_f, range_f
 
 
@@ -270,8 +270,8 @@ def check_all_nan(df, cols=None):
 
 
 DUPES_MSG = (
-    "{} contains duplicates, please specify group or additional filtering or select 'No Aggregation' from"
-    " Aggregation drop-down."
+    "The grouping [{}] contains duplicates, please specify group or additional filtering or select 'No Aggregation' "
+    "from Aggregation drop-down."
 )
 LIMIT_MSG = "Dataset exceeds {} records, cannot render. Please apply filter..."
 
@@ -363,35 +363,54 @@ def build_agg_data(df, x, y, inputs, agg, z=None, group_col=None, animate_by=Non
         idx_cols += make_list(y)
         agg_cols = make_list(z)
 
-    groups = df.groupby(idx_cols)
-    if agg in ["pctsum", "pctct"]:
-        func = "sum" if agg == "pctsum" else "size"
-        subidx_cols = [c for c in idx_cols if c not in make_list(group_col)]
-        groups = getattr(groups[agg_cols], func)()
-        groups = groups / getattr(df.groupby(subidx_cols)[agg_cols], func)() * 100
-        if len(agg_cols) > 1:
-            groups.columns = agg_cols
-        elif len(agg_cols) == 1:
-            groups.name = agg_cols[0]
-        code = (
-            "chart_data = chart_data.groupby(['{cols}'])[['{agg_cols}']].{agg}()\n"
-            "chart_data = chart_data / chart_data.groupby(['{subidx_cols}']).{agg}()\n"
-            "chart_data = chart_data.reset_index()"
-        )
-        code = code.format(
-            cols="', '".join(idx_cols),
-            subidx_cols="', '".join(subidx_cols),
-            agg_cols="', '".join(make_list(agg_cols)),
-            agg=func,
-        )
-        code = [code]
-    else:
-        groups = getattr(groups[agg_cols], agg)()
-        code = [
-            "chart_data = chart_data.groupby(['{cols}'])[['{agg_cols}']].{agg}().reset_index()".format(
-                cols="', '".join(idx_cols), agg_cols="', '".join(agg_cols), agg=agg
+    if agg == "drop_duplicates":
+        groups = [df[idx_cols + [col]].drop_duplicates() for col in agg_cols]
+        if len(groups) == 1:
+            groups = groups[0]
+            code = "chart_data = chart_data[['{}']].drop_duplicates()".format(
+                "','".join(idx_cols + agg_cols)
             )
-        ]
+        else:
+            groups = pd.merge(*groups, on=idx_cols, how="outer")
+            code = (
+                "idx_cols = ['{}']\n"
+                "agg_cols = ['{}']\n"
+                "chart_data = pd.merge(\n"
+                "\t*[chart_data[idx_cols + [col]].drop_duplicates() for col in agg_cols],\n"
+                "\ton=idx_cols,\n"
+                "\thow='outer'\n"
+                ")"
+            ).format("','".join(idx_cols), "','".join(agg_cols))
+    else:
+        groups = df.groupby(idx_cols)
+        if agg in ["pctsum", "pctct"]:
+            func = "sum" if agg == "pctsum" else "size"
+            subidx_cols = [c for c in idx_cols if c not in make_list(group_col)]
+            groups = getattr(groups[agg_cols], func)()
+            groups = groups / getattr(df.groupby(subidx_cols)[agg_cols], func)() * 100
+            if len(agg_cols) > 1:
+                groups.columns = agg_cols
+            elif len(agg_cols) == 1:
+                groups.name = agg_cols[0]
+            code = (
+                "chart_data = chart_data.groupby(['{cols}'])[['{agg_cols}']].{agg}()\n"
+                "chart_data = chart_data / chart_data.groupby(['{subidx_cols}']).{agg}()\n"
+                "chart_data = chart_data.reset_index()"
+            )
+            code = code.format(
+                cols="', '".join(idx_cols),
+                subidx_cols="', '".join(subidx_cols),
+                agg_cols="', '".join(make_list(agg_cols)),
+                agg=func,
+            )
+            code = [code]
+        else:
+            groups = getattr(groups[agg_cols], agg)()
+            code = [
+                "chart_data = chart_data.groupby(['{cols}'])[['{agg_cols}']].{agg}().reset_index()".format(
+                    cols="', '".join(idx_cols), agg_cols="', '".join(agg_cols), agg=agg
+                )
+            ]
     if animate_by is not None:
         full_idx = pd.MultiIndex.from_product(
             [df[c].unique() for c in idx_cols], names=idx_cols
@@ -447,6 +466,11 @@ def build_base_chart(
         raw_data, x, y, kwargs.get("z"), group_col, animate_by, group_val=group_val
     )
     x_col = str("x")
+    if x is None:
+        x = x_col
+        data.loc[:, x_col] = range(
+            1, len(data) + 1
+        )  # sequential integers: 1, 2, ..., N
     y_cols = make_list(y)
     z_col = kwargs.get("z")
     z_cols = make_list(z_col)
@@ -587,7 +611,7 @@ def build_base_chart(
     dupe_cols = main_group + (y_cols if len(z_cols) else [])
     check_exceptions(
         data[dupe_cols].rename(columns={x_col: x}),
-        allow_duplicates or agg == "raw",
+        allow_duplicates or agg in ["raw", "drop_duplicates"],
         unlimited_data=unlimited_data,
         data_limit=40000 if len(z_cols) or animate_by is not None else 15000,
     )
@@ -650,5 +674,5 @@ def weekday_tick_handler(col_data, col):
 def find_group_vals(df, group_cols):
     group_vals, _ = retrieve_chart_data(df, group_cols)
     group_vals = group_vals.drop_duplicates().sort_values(group_cols)
-    group_f, _ = build_formatters(group_vals)
+    group_f, _ = build_formatters(group_vals, nan_display="NaN")
     return group_f.format_dicts(group_vals.itertuples())
