@@ -4,7 +4,13 @@ from logging import getLogger
 
 import click
 
-from dtale.cli.loaders import arctic_loader, csv_loader, json_loader, parquet_loader
+from dtale.cli.loaders import (
+    arctic_loader,
+    csv_loader,
+    json_loader,
+    parquet_loader,
+    sqlite_loader,
+)
 
 logger = getLogger(__name__)
 
@@ -79,6 +85,7 @@ LOADERS = {
     csv_loader.LOADER_KEY: csv_loader,
     json_loader.LOADER_KEY: json_loader,
     parquet_loader.LOADER_KEY: parquet_loader,
+    sqlite_loader.LOADER_KEY: sqlite_loader,
 }
 
 
