@@ -1,13 +1,14 @@
+# coding=utf-8
 import random
+import six
 import string
 import strsimpy
-import struct
 import time
+import unicodedata
 
 import numpy as np
 import pandas as pd
 from scipy.stats import mstats
-from six import string_types
 from sklearn.preprocessing import (
     LabelEncoder,
     OrdinalEncoder,
@@ -16,10 +17,11 @@ from sklearn.preprocessing import (
     RobustScaler,
 )
 from sklearn.feature_extraction import FeatureHasher
+from string import printable
 from strsimpy.jaro_winkler import JaroWinkler
 
 import dtale.global_state as global_state
-from dtale.utils import classify_type
+from dtale.utils import classify_type, apply
 
 
 class ColumnBuilder(object):
@@ -49,6 +51,10 @@ class ColumnBuilder(object):
             self.builder = StandardizedColumnBuilder(name, cfg)
         elif column_type == "encoder":
             self.builder = EncoderColumnBuilder(name, cfg)
+        elif column_type == "cleaning":
+            self.builder = CleaningColumnBuilder(name, cfg)
+        elif column_type == "diff":
+            self.builder = DiffColumnBuilder(name, cfg)
         else:
             raise NotImplementedError(
                 "'{}' column builder not implemented yet!".format(column_type)
@@ -297,8 +303,8 @@ class RandomColumnBuilder(object):
             )
 
         # floats
-        low = self.cfg.get("low", 0)
-        high = self.cfg.get("high", 1)
+        low = float(self.cfg.get("low", 0))
+        high = float(self.cfg.get("high", 1))
         return pd.Series(
             np.random.uniform(low, high, len(data)), index=data.index, name=self.name
         )
@@ -313,7 +319,7 @@ class RandomColumnBuilder(object):
                 kwargs.append("chars='{chars}'".format(chars=self.cfg.get("chars")))
             kwargs = ", ".join(kwargs)
             return (
-                "import number\nimport random\n\n"
+                "\nimport number\nimport random\n\n"
                 "def id_generator(size=1500, chars=string.ascii_uppercase + string.digits):\n"
                 "\treturn ''.join(random.choice(chars) for _ in range(size))\n\n"
                 "df.loc[:, '{name}'] = pd.Series([id_generator({kwargs}) for _ in range(len(df)], index=df.index)"
@@ -374,7 +380,7 @@ class RandomColumnBuilder(object):
             low = self.cfg.get("low", 0)
             high = self.cfg.get("high", 100)
             return (
-                "import numpy as np\n\n"
+                "\nimport numpy as np\n\n"
                 "df.loc[:, '{name}'] = pd.Series(np.random.randint({low}, high={high}, size=len(df)), "
                 "index=df.index)"
             ).format(name=self.name, low=low, high=high)
@@ -382,15 +388,11 @@ class RandomColumnBuilder(object):
         low = self.cfg.get("low", 0)
         high = self.cfg.get("high", 1)
         return (
-            "import numpy as np\n\n"
+            "\nimport numpy as np\n\n"
             "df.loc[:, '{name}'] = pd.Series(np.random.uniform({low}, {high}, len(df)), index=df.index)"
         ).format(low=low, high=high, name=self.name)
 
 
-def float_to_hex(f):
-    return hex(struct.unpack("<I", struct.pack("<f", f))[0])
-
-
 class TypeConversionColumnBuilder(object):
     def __init__(self, name, cfg):
         self.name = name
@@ -413,10 +415,22 @@ class TypeConversionColumnBuilder(object):
                     pd.to_datetime(s, **date_kwargs), name=self.name, index=s.index
                 )
             elif to_type == "int":
+                if s.str.startswith("0x").any():
+
+                    def str_hex_to_int(v):
+                        return v if pd.isnull(v) else int(v, base=16)
+
+                    return pd.Series(
+                        apply(s, str_hex_to_int), name=self.name, index=s.index
+                    )
                 return pd.Series(
                     s.astype("float").astype("int"), name=self.name, index=s.index
                 )
             elif to_type == "float":
+                if s.str.startswith("0x").any():
+                    return pd.Series(
+                        apply(s, float.fromhex), name=self.name, index=s.index
+                    )
                 return pd.Series(
                     pd.to_numeric(s, errors="coerce"), name=self.name, index=s.index
                 )
@@ -433,14 +447,14 @@ class TypeConversionColumnBuilder(object):
                         def _process_mixed_bool(v):
                             if isinstance(v, bool):
                                 return v
-                            if isinstance(v, string_types):
+                            if isinstance(v, six.string_types):
                                 return dict(true=True, false=False).get(
                                     v.lower(), np.nan
                                 )
                             return np.nan
 
                         return pd.Series(
-                            s.apply(_process_mixed_bool), name=self.name, index=s.index
+                            apply(s, _process_mixed_bool), name=self.name, index=s.index
                         )
                 return pd.Series(s.astype(to_type), name=self.name, index=s.index)
         elif classifier == "I":  # date, float, category, str, bool
@@ -448,7 +462,7 @@ class TypeConversionColumnBuilder(object):
                 unit = self.cfg.get("unit") or "D"
                 if unit == "YYYYMMDD":
                     return pd.Series(
-                        s.astype("str").apply(pd.Timestamp),
+                        apply(s.astype("str"), pd.Timestamp),
                         name=self.name,
                         index=s.index,
                     )
@@ -456,11 +470,15 @@ class TypeConversionColumnBuilder(object):
                     pd.to_datetime(s, unit=unit), name=self.name, index=s.index
                 )
             elif to_type == "hex":
-                return pd.Series(s.apply(float_to_hex), name=self.name, index=s.index)
+
+                def int_to_hex(v):
+                    return v if pd.isnull(v) else hex(v)
+
+                return pd.Series(apply(s, int_to_hex), name=self.name, index=s.index)
             return pd.Series(s.astype(to_type), name=self.name, index=s.index)
         elif classifier == "F":  # str, int
             if to_type == "hex":
-                return pd.Series(s.apply(float_to_hex), name=self.name, index=s.index)
+                return pd.Series(apply(s, float.hex), name=self.name, index=s.index)
             return pd.Series(s.astype(to_type), name=self.name, index=s.index)
         elif classifier == "D":  # str, int
             if to_type == "int":
@@ -472,7 +490,7 @@ class TypeConversionColumnBuilder(object):
                         index=s.index,
                     )
                 return pd.Series(
-                    s.apply(lambda x: time.mktime(x.timetuple())).astype(int)
+                    apply(s, lambda x: time.mktime(x.timetuple())).astype(int)
                 )
             return pd.Series(
                 s.dt.strftime(self.cfg.get("fmt") or "%Y%m%d"),
@@ -485,14 +503,6 @@ class TypeConversionColumnBuilder(object):
             "data type conversion not supported for dtype: {}".format(from_type)
         )
 
-    def build_hex_code(self, s):
-        return (
-            "import struct\n\n"
-            "def float_to_hex(f):\n"
-            "\treturn hex(struct.unpack('<I', struct.pack('<f', f))[0])\n\n"
-            "pd.Series({s}.apply(float_to_hex), name='{name}', index={s}.index)"
-        ).format(s=s, name=self.name)
-
     def build_inner_code(self):
         col, from_type, to_type = (self.cfg.get(p) for p in ["col", "from", "to"])
         s = "df['{col}']".format(col=col)
@@ -506,13 +516,25 @@ class TypeConversionColumnBuilder(object):
                 code = "pd.Series(pd.to_datetime({s}, {kwargs}), name='{name}', index={s}.index)"
                 return code.format(s=s, name=self.name, kwargs=date_kwargs)
             elif to_type == "int":
-                return "pd.Series({s}.astype('float').astype('int'), name='{name}', index={s}.index)".format(
-                    s=s, name=self.name
-                )
+                return (
+                    "s = {s}"
+                    "if s.str.startswith('0x').any():\n"
+                    "\tdef str_hex_to_int(v):\n"
+                    "\t\treturn v if pd.isnull(v) else int(v, base=16)\n"
+                    "\tstr_data = s.apply(str_hex_to_int)\n"
+                    "else:\n"
+                    "\tstr_data = s.astype('float').astype('int')\n"
+                    "pd.Series(str_data, name='{name}', index=s.index)"
+                ).format(s=s, name=self.name)
             elif to_type == "float":
-                return "pd.Series(pd.to_numeric({s}, errors='coerce'), name='{name}', index={s}.index)".format(
-                    s=s, name=self.name
-                )
+                return (
+                    "s = {s}"
+                    "if s.str.startswith('0x').any():\n"
+                    "\tstr_data = s.apply(float.fromhex)\n"
+                    "else:\n"
+                    "\tstr_data = pd.to_numeric(s, errors='coerce')\n"
+                    "pd.Series(str_data, name='{name}', index=s.index)"
+                ).format(s=s, name=self.name)
             else:
                 if from_type.startswith("mixed"):
                     if to_type == "float":
@@ -545,13 +567,19 @@ class TypeConversionColumnBuilder(object):
                     s=s, name=self.name, unit=unit
                 )
             elif to_type == "hex":
-                return self.build_hex_code(s)
+                return (
+                    "pd.Series(\n"
+                    "\t{s}.apply(lambda v: v if pd.isnull(v) else hex(v)), name='{name}', index={s}.index\n"
+                    ")"
+                ).format(s=s, name=self.name)
             return "pd.Series({s}.astype('{to_type}'), name='{name}', index={s}.index)".format(
                 s=s, to_type=to_type, name=self.name
             )
         elif classifier == "F":  # str, int, hex
             if to_type == "hex":
-                return self.build_hex_code(s)
+                return "pd.Series(s.apply(float.hex), name='{name}', index={s}.index)".format(
+                    s=s, name=self.name
+                )
             return "pd.Series(s.astype('{to_type}'), name='{name}', index={s}.index)".format(
                 s=s, to_type=to_type, name=self.name
             )
@@ -646,7 +674,7 @@ class WinsorizeColumnBuilder(object):
             winsorize_params = ""
         if len(group or []):
             code = (
-                "from scipy.stats import mstats\n\n"
+                "\nfrom scipy.stats import mstats\n\n"
                 "def winsorize_series(group):\n"
                 "\treturn mstats.winsorize(group{params})\n\n"
                 "winsorized_data = data.groupby(['{group}'])['{col}'].transform(winsorize_series)\n"
@@ -659,7 +687,7 @@ class WinsorizeColumnBuilder(object):
             )
         else:
             code = (
-                "from scipy.stats import mstats\n\n"
+                "\nfrom scipy.stats import mstats\n\n"
                 "winsorized_data = mstats.winsorize(data['{col}']{params})\n"
                 "winsorized_data = pd.Series(winsorized_data, index=data.index, name='{name}')"
             ).format(params=winsorize_params, col=col, name=self.name)
@@ -733,10 +761,10 @@ class SimilarityColumnBuilder(object):
             similarity = strsimpy.jaccard.Jaccard(int(self.cfg.get("k", 3)))
             if normalized:
                 similarity = SimilarityNormalizeWrapper(similarity)
-        distances = (
-            data[[left_col, right_col]]
-            .fillna("")
-            .apply(lambda rec: similarity.distance(*rec), axis=1)
+        distances = apply(
+            data[[left_col, right_col]].fillna(""),
+            lambda rec: similarity.distance(*rec),
+            axis=1,
         )
         return pd.Series(distances, index=data.index, name=self.name)
 
@@ -897,3 +925,230 @@ class EncoderColumnBuilder(object):
                 "\tdf.loc[:, str(new_col.name)] = new_col"
             ).format(n=self.cfg.get("n"), col=col)
         return ""
+
+
+def clean(s, cleaner, cfg):
+    if cleaner == "drop_multispace":
+        return s.str.replace(r"[ ]+", " ")
+    elif cleaner == "drop_punctuation":
+        if six.PY3:
+            return apply(
+                s, lambda x: x.translate(str.maketrans("", "", string.punctuation))
+            )
+        return apply(s, lambda x: x.translate(None, string.punctuation))
+    elif cleaner == "stopwords":
+        stopwords = cfg.get("stopwords") or []
+
+        def clean_stopwords(x):
+            return " ".join([w for w in x.split(" ") if w not in stopwords])
+
+        return apply(s, clean_stopwords)
+    elif cleaner == "nltk_stopwords":
+        language = cfg.get("language") or "english"
+        try:
+            import nltk
+
+            nltk.download("stopwords")
+            nltk.download("punkt")
+
+            nltk_stopwords_set = set(nltk.corpus.stopwords.words(language))
+
+            def clean_nltk_stopwords(x):
+                return " ".join(
+                    [
+                        w
+                        for w in nltk.tokenize.word_tokenize(x)
+                        if w not in nltk_stopwords_set
+                    ]
+                )
+
+            return apply(s.fillna(""), clean_nltk_stopwords)
+        except ImportError:
+            raise Exception(
+                "You must install the 'nltk' package in order to use this cleaner!"
+            )
+    elif cleaner == "drop_numbers":
+        return s.str.replace(r"[0-9]+", "")
+    elif cleaner == "keep_alpha":
+        return apply(s, lambda x: "".join(c for c in x if c.isalpha()))
+    elif cleaner == "normalize_accents":
+        return apply(
+            s,
+            lambda x: unicodedata.normalize("NFKD", u"{}".format(x))
+            .encode("ASCII", "ignore")
+            .decode("utf-8"),
+        )
+    elif cleaner == "drop_all_space":
+        return s.str.replace(r"[ ]+", "")
+    elif cleaner == "drop_repeated_words":
+
+        def drop_repeats(val):
+            def _load():
+                val_segs = val.split(" ")
+                for i, v2 in enumerate(val_segs):
+                    if i == 0:
+                        yield v2
+                    elif val_segs[i - 1] != v2:
+                        yield v2
+
+            return " ".join(list(_load()))
+
+        return apply(s, drop_repeats)
+    elif cleaner == "add_word_number_space":
+        return s.str.replace(r"(\d+(\.\d+)?)", r" \1 ")
+    elif cleaner == "drop_repeated_chars":
+
+        def drop_repeats(val):
+            def _load():
+                for i, v2 in enumerate(val):
+                    if i == 0:
+                        yield v2
+                    elif val[i - 1] != v2:
+                        yield v2
+
+            return "".join(list(_load()))
+
+        return apply(s, drop_repeats)
+    elif cleaner == "update_case":
+        case = cfg.get("caseType")
+        return getattr(s.str, case)()
+    elif cleaner == "space_vals_to_empty":
+        return s.str.replace(r"[ ]+", "")
+    elif cleaner == "underscore_to_space":
+        return s.str.replace("_", " ")
+    elif cleaner == "hidden_chars":
+        return s.str.replace(r"[^{}]+".format(printable), "")
+    elif cleaner == "replace_hyphen_w_space":
+        return s.str.replace(r"[‐᠆﹣－⁃−-]+", " ")
+    return s
+
+
+def clean_code(cleaner, cfg):
+    if cleaner == "drop_multispace":
+        return ["s = s.str.replace(r'[ ]+', ' ')"]
+    elif cleaner == "drop_punctuation":
+        if six.PY3:
+            return [
+                "s = s.apply(lambda x: x.translate(str.maketrans('', '', string.punctuation))"
+            ]
+        return ["s = s.apply(lambda x: x.translate(None, string.punctuation))"]
+    elif cleaner == "stopwords":
+        stopwords = cfg.get("stopwords") or []
+        return [
+            "def clean_stopwords(x):",
+            "\treturn ' '.join([w for w in x.split(' ') if w not in ['{}'])".format(
+                "','".join(stopwords)
+            ),
+            "s = s.apply(clean_stopwords)",
+        ]
+    elif cleaner == "nltk_stopwords":
+        return [
+            "import nltk\n",
+            "nltk.download('stopwords')" "nltk.download('punkt')\n",
+            "nltk_stopwords_set = set(nltk.corpus..words('{}'))\n".format(
+                cfg.get("language") or "english"
+            ),
+            "def clean_nltk_stopwords(x):",
+            "\treturn ' '.join(",
+            "\t\t[w for w in nltk.tokenize.word_tokenize(x) if w not in nltk_stopwords_set]",
+            "\t)",
+            "s = s.apply(clean_nltk_stopwords)",
+        ]
+    elif cleaner == "drop_numbers":
+        return ["""s = s.str.replace(r'[0-9]+', '')"""]
+    elif cleaner == "keep_alpha":
+        return ["s = s.apply(lambda x: ''.join(c for c in x if c.isalpha()))"]
+    elif cleaner == "normalize_accents":
+        return [
+            "import unicodedata\n",
+            "s = s.apply(",
+            "\tlambda x: unicodedata.normalize('NFKD', u'{}'.format(x)).encode('ASCII', 'ignore').decode('utf-8')",
+            ")",
+        ]
+    elif cleaner == "drop_all_space":
+        return ["s.str.replace(r'[ ]+', '')"]
+    elif cleaner == "drop_repeated_words":
+        return [
+            "def drop_repeated_words(val):",
+            "\tdef _load():",
+            "\t\tval_segs = val.split(' ')",
+            "\t\t\tfor i, v2 in enumerate(val_segs):",
+            "\t\t\t\tif i == 0:",
+            "\t\t\t\t\tyield v2",
+            "\t\t\t\telif val_segs[i - 1] != v2:",
+            "\t\t\t\t\tyield v2",
+            "\treturn ' '.join(list(_load()))",
+            "s = s.apply(drop_repeated_words)",
+        ]
+    elif cleaner == "add_word_number_space":
+        return ["s = s.str.replace(r'(\\d+(\\.\\d+)?)', r' \\1 ')"]
+    elif cleaner == "drop_repeated_chars":
+        return [
+            "def drop_repeated_chars(val):",
+            "\tdef _load():",
+            "\t\tfor i, v2 in enumerate(val):",
+            "\t\t\tif i == 0:",
+            "\t\t\t\tyield v2",
+            "\t\t\telif val[i - 1] != v2:",
+            "\t\t\t\tyield v2",
+            "\treturn ''.join(list(_load()))",
+            "s = s.apply(drop_repeated_chars)",
+        ]
+    elif cleaner == "update_case":
+        case = cfg.get("caseType")
+        return ["s = s.str.{}()".format(case)]
+    elif cleaner == "space_vals_to_empty":
+        return ["s = s.str.replace(r'[ ]+', '')"]
+    elif cleaner == "underscore_to_space":
+        return ["s = s.str.replace('_', ' ')"]
+    elif cleaner == "hidden_chars":
+        return [
+            "from string import printable",
+            "s = s.str.replacer(r'[^{}]+'.format(printable), '')",
+        ]
+    elif cleaner == "replace_hyphen_w_space":
+        return ("s = s.str.replacer(s.str.replace(r'[‐᠆﹣－⁃−-]+', ' ')",)
+    return []
+
+
+class CleaningColumnBuilder(object):
+    def __init__(self, name, cfg):
+        self.name = name
+        self.cfg = cfg
+
+    def build_column(self, data):
+        col, cleaners = (self.cfg.get(p) for p in ["col", "cleaners"])
+        s = data[col]
+        for cleaner in cleaners:
+            s = clean(s, cleaner, self.cfg)
+        return s
+
+    def build_code(self):
+        col, cleaners = (self.cfg.get(p) for p in ["col", "cleaners"])
+        code = ["s= ddf['{col}']".format(col=col)]
+        for cleaner in cleaners:
+            code += clean_code(cleaner, self.cfg)
+        code.append(
+            "df.loc[:, '{name}'] = pd.Series(s, index=df.index, name='{name}')".format(
+                name=self.name
+            )
+        )
+        return code
+
+
+class DiffColumnBuilder(object):
+    def __init__(self, name, cfg):
+        self.name = name
+        self.cfg = cfg
+
+    def build_column(self, data):
+        col, periods = (self.cfg.get(p) for p in ["col", "periods"])
+        return pd.Series(data[col].diff(int(periods)), index=data.index, name=self.name)
+
+    def build_code(self):
+        col, periods = (self.cfg.get(p) for p in ["col", "periods"])
+        return "df.loc[:, '{name}'] = pd.Series(df['{col}'].diff({periods}), index=df.index, name='{name}')".format(
+            name=self.name,
+            col=col,
+            periods=periods,
+        )
