@@ -82,6 +82,9 @@ def build_show_options(options=None):
         allow_cell_edits=True,
         inplace=False,
         drop_index=False,
+        precision=2,
+        show_columns=None,
+        hide_columns=None,
     )
     config_options = {}
     config = get_config()
@@ -119,6 +122,19 @@ def build_show_options(options=None):
         config_options["drop_index"] = get_config_val(
             config, defaults, "drop_index", "getboolean"
         )
+        config_options["precision"] = get_config_val(
+            config, defaults, "precision", "getint"
+        )
+        config_options["show_columns"] = get_config_val(
+            config, defaults, "show_columns"
+        )
+        if config_options["show_columns"]:
+            config_options["show_columns"] = config_options["show_columns"].split(",")
+        config_options["hide_columns"] = get_config_val(
+            config, defaults, "hide_columns"
+        )
+        if config_options["hide_columns"]:
+            config_options["hide_columns"] = config_options["hide_columns"].split(",")
 
     return dict_merge(defaults, config_options, options)
 
