@@ -1,5 +1,3 @@
 from .Wordcloud import Wordcloud
 
-__all__ = [
-    "Wordcloud"
-]
+__all__ = ["Wordcloud"]
