@@ -18,8 +18,8 @@ from dtale.dash_application.layout.utils import (
     build_selections,
     AGGS,
 )
+from dtale.query import build_query
 from dtale.utils import (
-    build_query,
     classify_type,
     dict_merge,
     find_dtype,
@@ -27,9 +27,9 @@ from dtale.utils import (
     json_date,
     json_float,
     make_list,
-    run_query,
     get_dtypes,
 )
+from dtale.query import run_query
 from dtale.charts.utils import (
     MAX_GROUPS,
     ZAXIS_CHARTS,
@@ -56,7 +56,7 @@ def build_histogram(data_id, col, query, point_filter):
         query,
         global_state.get_context_variables(data_id),
     )
-    data = data.query(build_group_inputs_filter(data, [point_filter]))
+    data = run_query(data, build_group_inputs_filter(data, [point_filter]))
     s = data[~pd.isnull(data[col])][col]
     hist_data, hist_labels = np.histogram(s, bins=10)
     hist_labels = list(map(lambda x: json_float(x, precision=3), hist_labels[1:]))
