@@ -9,6 +9,7 @@ import plotly
 from pkg_resources import parse_version
 
 import dtale.dash_application.custom_geojson as custom_geojson
+import dtale.global_state as global_state
 from dtale.charts.utils import YAXIS_CHARTS, ZAXIS_CHARTS, find_group_vals
 from dtale.dash_application.layout.utils import (
     build_input,
@@ -21,7 +22,7 @@ from dtale.dash_application.layout.utils import (
     FREQS,
     FREQ_LABELS,
 )
-from dtale.query import inner_build_query
+from dtale.query import build_query, inner_build_query, run_query
 from dtale.utils import (
     ChartBuildingError,
     classify_type,
@@ -47,7 +48,7 @@ def base_layout(app_root, **kwargs):
     :rtype: str
     """
     webroot_html = ""
-    favicon_path = "../../images/favicon.png"
+    favicon_path = "../../dtale/static/images/favicon.png"
     if is_app_root_defined(app_root):
         webroot_html = """
         <script type="text/javascript">
@@ -56,7 +57,7 @@ def base_layout(app_root, **kwargs):
         """.format(
             app_root=app_root
         )
-        favicon_path = "{}/images/favicon.png".format(app_root)
+        favicon_path = "{}/dtale/static/images/favicon.png".format(app_root)
     return """
         <!DOCTYPE html>
         <html>
@@ -233,7 +234,7 @@ PROJECTIONS = [
 
 
 def build_img_src(proj, img_type="projections"):
-    return "../images/{}/{}.png".format(img_type, "_".join(proj.split(" ")))
+    return "../static/images/{}/{}.png".format(img_type, "_".join(proj.split(" ")))
 
 
 def build_proj_hover_children(proj):
@@ -398,7 +399,8 @@ YLORRD = [
     "#bd0026",
     "#800026",
 ]
-DEFAULT_CSALES = {"heatmap": JET, "maps": REDS, "3d_Scatter": YLORRD}
+YLGRBL = ["#ffffd9", "#d5efb3", "#73c9bc", "#1b99c2", "#1c4ea2", "#081d58"]
+DEFAULT_CSALES = {"heatmap": JET, "maps": REDS, "3d_Scatter": YLORRD, "surface": YLGRBL}
 ANIMATION_CHARTS = ["line"]
 ANIMATE_BY_CHARTS = ["bar", "3d_scatter", "heatmap", "maps"]
 
@@ -649,7 +651,7 @@ def bar_input_style(**inputs):
 def colorscale_input_style(**inputs):
     return dict(
         display="block"
-        if inputs.get("chart_type") in ["heatmap", "maps", "3d_scatter"]
+        if inputs.get("chart_type") in ["heatmap", "maps", "3d_scatter", "surface"]
         else "none"
     )
 
@@ -670,7 +672,7 @@ def animate_styles(df, **inputs):
 
 def lock_zoom_style(chart_type):
     return (
-        dict(display="block")
+        dict(display="block", textTransform="none")
         if chart_type in ["3d_scatter", "surface"]
         else dict(display="none")
     )
@@ -750,13 +752,44 @@ def build_map_type_tabs(map_type):
     )
 
 
+def build_hoverable(content, hoverable_content):
+    return html.Div(
+        [
+            content,
+            html.Div(
+                hoverable_content,
+                className="hoverable__content map-types",
+                style=dict(top="50%"),
+            ),
+        ],
+        style=dict(borderBottom="none"),
+        className="hoverable",
+    )
+
+
 def main_inputs_and_group_val_display(inputs):
     if show_group_input(inputs):
         return dict(display="block"), "col-md-8"
     return dict(display="none"), "col-md-12"
 
 
-def charts_layout(df, settings, **inputs):
+def build_slider_counts(df, data_id, query_value):
+    record_ct = len(
+        run_query(
+            df,
+            build_query(data_id, query_value),
+            global_state.get_context_variables(data_id),
+        )
+    )
+    slider_counts = {
+        v * 20: {"label": "{}% ({:,.0f})".format(v * 20, (v * 2) / 10 * record_ct)}
+        for v in range(1, 6)
+    }
+    slider_counts[100]["style"] = {"white-space": "nowrap"}
+    return slider_counts
+
+
+def charts_layout(df, settings, data_id, **inputs):
     """
     Builds main dash inputs with dropdown options populated with the columns of the dataframe associated with the
     page. Inputs included are: chart tabs, query, x, y, z, group, aggregation, rolling window/computation,
@@ -766,10 +799,12 @@ def charts_layout(df, settings, **inputs):
     :type df: :class:`pandas:pandas.DataFrame`
     :param settings: global settings associated with this dataframe (contains properties like "query")
     :type param: dict
+    :param data_id: identifier for the data we are viewing
+    :type data_id: string
     :return: dash markup
     """
-    chart_type, x, y, z, group, agg = (
-        inputs.get(p) for p in ["chart_type", "x", "y", "z", "group", "agg"]
+    chart_type, x, y, z, group, agg, load = (
+        inputs.get(p) for p in ["chart_type", "x", "y", "z", "group", "agg", "load"]
     )
     y = y or []
     show_input = show_input_handler(chart_type)
@@ -793,6 +828,7 @@ def charts_layout(df, settings, **inputs):
     query_value = inputs.get("query") or inner_build_query(
         settings, settings.get("query")
     )
+
     query_label = html.Div(
         [
             html.Span("Query"),
@@ -946,6 +982,38 @@ def charts_layout(df, settings, **inputs):
             ),
             className="row pt-3 pb-3 charts-filters",
         ),
+        html.Div(
+            html.Div(
+                [
+                    html.Div(
+                        [
+                            build_hoverable(
+                                html.Span("Load"),
+                                html.Span(
+                                    "Load a random percentage of rows from your dataset."
+                                ),
+                            ),
+                            dcc.Slider(
+                                id="load-input",
+                                min=10,
+                                max=100,
+                                step=10,
+                                value=100 if load is None else load,
+                                className="w-100",
+                                marks=build_slider_counts(df, data_id, query_value),
+                                tooltip={
+                                    "always_visible": False,
+                                    "placement": "left",
+                                },
+                            ),
+                        ],
+                        className="input-group mr-3",
+                    )
+                ],
+                className="col",
+            ),
+            className="row pt-3 pb-3 charts-filters",
+        ),
         html.Div(
             [
                 html.Div(
@@ -957,7 +1025,7 @@ def charts_layout(df, settings, **inputs):
                                     dcc.Dropdown(
                                         id="x-dropdown",
                                         options=x_options,
-                                        placeholder="Select a column",
+                                        placeholder="Default Index (1, 2, ..., N)",
                                         value=x,
                                         style=dict(width="inherit"),
                                     ),
@@ -1308,7 +1376,6 @@ def charts_layout(df, settings, **inputs):
                                         options=[
                                             build_option(v, AGGS[v])
                                             for v in [
-                                                "raw",
                                                 "count",
                                                 "nunique",
                                                 "sum",
@@ -1317,6 +1384,7 @@ def charts_layout(df, settings, **inputs):
                                                 "corr",
                                                 "first",
                                                 "last",
+                                                "drop_duplicates",
                                                 "median",
                                                 "min",
                                                 "max",
@@ -1328,7 +1396,7 @@ def charts_layout(df, settings, **inputs):
                                                 "pctct",
                                             ]
                                         ],
-                                        placeholder="Select an aggregation",
+                                        placeholder="No Aggregation",
                                         style=dict(width="inherit"),
                                         value=agg or "raw",
                                     ),
