@@ -28,6 +28,7 @@ from dtale.dash_application.layout.layout import (
     build_candlestick_options,
     build_mapbox_style_options,
     build_proj_hover_children,
+    build_slider_counts,
     build_treemap_options,
     charts_layout,
     colorscale_input_style,
@@ -56,14 +57,14 @@ class DtaleDash(dash.Dash):
     def __init__(self, *args, **kwargs):
         server = kwargs.get("server")
         kwargs["external_stylesheets"] = [
-            "/css/main.css",
-            "/css/dash.css",
-            "/css/github_fork.css",
+            "/dtale/static/css/main.css",
+            "/dtale/static/css/dash.css",
+            "/dtale/static/css/github_fork.css",
         ]
         kwargs["external_scripts"] = [
-            "/dash/components_bundle.js",
-            "/dash/custom_bundle.js",
-            "/dist/base_styles_bundle.js",
+            "/dtale/static/dash/components_bundle.js",
+            "/dtale/static/dash/custom_bundle.js",
+            "/dtale/static/dist/base_styles_bundle.js",
         ]
 
         app_root = server.config.get("APPLICATION_ROOT")
@@ -136,11 +137,16 @@ def init_callbacks(dash_app):
             Output("query-data", "data"),
             Output("query-input", "style"),
             Output("query-input", "title"),
+            Output("load-input", "marks"),
         ],
         [Input("query-input", "value")],
-        [State("url", "pathname"), State("query-data", "data")],
+        [
+            State("url", "pathname"),
+            State("query-data", "data"),
+            State("load-input", "marks"),
+        ],
     )
-    def query_input(query, pathname, curr_query):
+    def query_input(query, pathname, curr_query, curr_marks):
         """
         dash callback for storing valid pandas dataframe queries.  This acts as an intermediary between values typed
         by the user and values that are applied to pandas dataframes.  Most of the time what the user has typed is not
@@ -158,13 +164,19 @@ def init_callbacks(dash_app):
             data_id = get_data_id(pathname)
             data = global_state.get_data(data_id)
             ctxt_vars = global_state.get_context_variables(data_id)
-            run_query(data, query, ctxt_vars)
-            return query, {"line-height": "inherit"}, ""
+            df = run_query(data, query, ctxt_vars)
+            return (
+                query,
+                {"line-height": "inherit"},
+                "",
+                build_slider_counts(df, data_id, query),
+            )
         except BaseException as ex:
             return (
                 curr_query,
                 {"line-height": "inherit", "background-color": "pink"},
                 str(ex),
+                curr_marks,
             )
 
     @dash_app.callback(
@@ -197,6 +209,7 @@ def init_callbacks(dash_app):
             Input("agg-dropdown", "value"),
             Input("window-input", "value"),
             Input("rolling-comp-dropdown", "value"),
+            Input("load-input", "value"),
         ],
         [State("url", "pathname"), State("query-data", "data")],
     )
@@ -212,6 +225,7 @@ def init_callbacks(dash_app):
         agg,
         window,
         rolling_comp,
+        load,
         pathname,
         query,
     ):
@@ -230,9 +244,10 @@ def init_callbacks(dash_app):
             z=z,
             group=group,
             group_val=group_val,
-            agg=agg,
+            agg=agg or "raw",
             window=window,
             rolling_comp=rolling_comp,
+            load=load,
         )
         data_id = get_data_id(pathname)
         options = build_input_options(global_state.get_data(data_id), **inputs)
@@ -872,7 +887,7 @@ def init_callbacks(dash_app):
         data_id = get_data_id(pathname)
         df = global_state.get_data(data_id)
         settings = global_state.get_settings(data_id) or {}
-        return charts_layout(df, settings, **params)
+        return charts_layout(df, settings, data_id, **params)
 
     custom_geojson.init_callbacks(dash_app)
     drilldown_modal.init_callbacks(dash_app)
