@@ -54,6 +54,14 @@ def get_data(data_id=None):
     return DATA.get(data_id)
 
 
+def build_data_id():
+    if not len(get_data()):
+        return "1"
+    current_ids = [int(data_id) for data_id in get_data().keys()]
+    new_id = max(current_ids) + 1
+    return str(new_id)
+
+
 def get_dataset(data_id=None):
     global DATASETS
 
@@ -78,6 +86,11 @@ def get_dtypes(data_id=None):
     return DTYPES.get(data_id)
 
 
+def get_dtype_info(data_id, col):
+    dtypes = get_dtypes(data_id)
+    return next((c for c in dtypes or [] if c["name"] == col), None)
+
+
 def get_settings(data_id=None):
     global SETTINGS
 
