@@ -195,7 +195,7 @@ def json_string(x, nan_display="", **kwargs):
     """
     if pd.isnull(x):
         return nan_display
-    if x or x in ["", False, 0]:
+    if x or x in ["", False, 0, pd.Timedelta(0)]:
         try:
             return str(x)
         except UnicodeEncodeError:
@@ -320,7 +320,7 @@ class JSONFormatter(object):
 
     def add_string(self, idx, name=None):
         def f(x, nan_display):
-            return json_string(x)
+            return json_string(x, nan_display=nan_display)
 
         self.fmts.append([idx, name, f])
 
@@ -345,7 +345,9 @@ class JSONFormatter(object):
 
     def add_timestamp(self, idx, name=None, as_string=False):
         def f(x, nan_display):
-            return json_timestamp(x, as_string=as_string or self.as_string)
+            return json_timestamp(
+                x, nan_display=nan_display, as_string=as_string or self.as_string
+            )
 
         self.fmts.append([idx, name, f])
 
@@ -384,8 +386,8 @@ class JSONFormatter(object):
         cols = [col for col in df.columns if col in formatters]
         return pd.concat(
             [
-                df[col].apply(
-                    lambda v: formatters[col](v, nan_display=self.nan_display)
+                apply(
+                    df[col], lambda v: formatters[col](v, nan_display=self.nan_display)
                 )
                 for col in cols
             ],
@@ -713,3 +715,12 @@ def fix_url_path(path):
     while "//" in path:
         path = path.replace("//", "/")
     return path
+
+
+def apply(df, func, *args, **kwargs):
+    try:
+        import swifter  # noqa: F401
+
+        return df.swifter.progress_bar(False).apply(func, *args, **kwargs)
+    except ImportError:
+        return df.apply(func, *args, **kwargs)
